ScreenInit:
	ldx #16					;We're going to copy 16 registers 
ScreenInitAgain:	
	dex
	lda VicScreenSettings,x	;Get A parameter
	sta $9000,X				;Store to the video registers at $9000
	txa
	bne ScreenInitAgain
	rts
	
VicScreenSettings:
	db $0C		;$9000 - horizontal centering
	db $26		;$9001 - vertical centering
	db $96		;$9002 - set # of columns / 
					;Bit7 = screen base bit ($16 for screen at $1000)
	db $AE		;$9003 - set # of rows
	db $7A		;$9004 - TV raster beam line
	db $F0		;$9005 - bits 0-3 start of character memory /  
					;bits 4-7 is rest of video address 
					;$(CF for screen at $1000)
	db $57		;$9006 - horizontal position of light pen
	db $EA		;$9007 - vertical position of light pen
	db $FF		;$9008 - Digitized value of paddle X
	db $FF		;$9009 - Digitized value of paddle Y
	db $00		;$900A - Frequency for oscillator 1 (low)
	db $00		;$900B - Frequency for oscillator 2 (medium)
	db $00		;$900C - Frequency for oscillator 3 (high)
	db $00		;$900D - Frequency of noise source
	db $00		;$900E - bit 0-3 sets volume of all sound / 
					;bits 4-7 are auxiliary color information
	db $00+8 	;$900F - Screen and border color register
	
slowdown:	;this exists for systems that run too fast
	rts

newline:
	lda #0		;new line
	sta z_d

	inc z_e

	lda z_e
	cmp #23			;at bottom of screen
	bcs lbl52779		;no? return, else cls!
	rts
lbl52779

cls:
	lda z_h
	pha
	lda z_l
	pha
	lda z_b
	pha
	lda z_c
	pha
		lda #0
		sta z_d			;x,y
		sta z_e
		sta z_l			;char,inc
		lda #$20
		sta z_h

		lda #22		;width,height
		sta z_b
		lda #$23
		sta z_c

		jsr fillarea
	pla
	sta	z_c
	pla
	sta z_b
	
	pla
	sta	z_l
	pla
	sta z_h
	rts


printnumber:		;print char a at loc de
	clc
	adc #48			;ascii 0

printchar:			;print char a with sprite routine
	cmp #0
	bne Notprintchar0
;printchar0:
	inc z_d
	rts
Notprintchar0:	
	
	tax
	pha
	;pushpair z_bc
	lda z_d
	pha
	lda z_e
	pha
	
	lda z_h
	pha
	lda z_l
	pha
		lda #$1E			;Screen base is $1E00
		sta z_h		
		
		lda z_d				;Xpos
		ifdef ScreenWidth32
			clc
			adc #4
		endif
		sta z_l

		ldy z_e				;Ypos
		beq GetVDPScreenPos_YZero
		
		lda z_l
GetVDPScreenPos_Addagain:	;Repeatedly add screen width (22) Y times 
		clc
		adc #22			;40 bytes per line

		bcc GetVDPScreenPos_AddagainB
		inc z_h		;Add Carry
GetVDPScreenPos_AddagainB		
		dey
		bne GetVDPScreenPos_Addagain
		sta z_l
GetVDPScreenPos_YZero:	
		txa
		clc
		cmp #64	
		bcs PrintChar_Letters
;	sbc #32
		jmp PrintChar_Symbols
PrintChar_Letters:
		sbc #64;
		and #%00011111
PrintChar_Symbols:
		ldx #0
		sta (z_hl,x)

		lda z_h
		adc #$78	;Colors at $9600 (add $7800 offset)
		sta z_h
		
		lda #3
		sta (z_hl,x)
		
	pla
	sta	z_l
	pla
	sta z_h
	pla
	sta	z_e
	pla
	sta z_d
	;pullpairsafe z_bc
	pla
	inc z_d
	rts

	
	
			
;We Return ---FRLDU in z_h for Player 0, and z_L for Player 1

ReadJoystick:	;Returns %--21RLDU
	lda z_h
	pha
		lda #%01111111
		sta $9122	;Set Data Direction of port B to READ (0=read)
		
	;	lda #%11000011
	;	sta $9113	;Set Data Direction of port A to READ (0=read)
		
		lda $9120	;Port B (R------- Switch)
		sta z_as
		
		lda #255	;Set all buttons to unpressed
		sta z_h
		
		lda $911F	;Port A (--FLDU-- Switches)
		rol
		rol
		rol
		rol z_h		;Shift in Fire
		rol z_as		
		rol z_h		;Shift in Right
		rol
		rol z_h		;Shift in Left
		rol
		rol z_h		;Shift in Down
		rol
		rol z_h		;Shift in Up
			
		ldx z_h
	pla
	sta z_h
	txa
	rts
	